/* ed2k_slist
*
* --- DESCRIPTION ---
*
* this simple command line tool has been written for use with the
* edonkey 2000 file-sharing program (www.edonkey2000.com)
*
* ed2k_slist scans the eDonkey2000 server.met file for a list of servers
* and removes those from the file which are no longer alive.
*
* Additionally, it can print out an html page with links to all those eDonkey
* servers which are alive in ed2k:// format.
*
* it assumes that it is run from the edonkey2000 program directory
* (which contains the server.met file)
*
*
* --- COPYRIGHT AND LICENCE ---
*
* Copyright (C) 2001 Tim-Philipp Mller (t.i.m@orange.net)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*/

/* --- DEFINE IF YOU COMPILE FOR 'LINUX' OR 'WINDOWS' here... !!! ---------------------------- */

#define LINUX

#define VERSION "0.2 alpha"

#include "global.h"

#ifdef LINUX
#include "linux_net.c"
#else
#include "win_net.c"
#endif

#ifdef WINDOWS
WSADATA	wsaData;
#endif

/* MAIN PROGRAM *****************************************************************************/

int main (int argc, char *argv[])
{
        int       i;
   		time_t    our_time;
		char	  *dummy;


        if (argc > 1) {
                for (i=1; i<argc; i++) {
                        if (strncmp(argv[i],"-s",2)==0) {
                                verbose = 0;                            /* silent mode   */
                                continue;                               /* next argument */
                        }
                        if (strncmp(argv[i],"-html",5)==0) {
                                htmloutput = 1;                         /* html output   */
								continue;
						}
						if (strncmp(argv[i],"-T",2)==0) {
							if (strlen(argv[i])<=2) {
								graceful_exit ("Timeout value not found. Must follow WITHOUT space right after\n"
											   "switch, e.g. '-T45' sets timeout to 45 seconds...\n");
							} else {
								SERVER_TIMEOUT = strtol (argv[i]+2, &dummy, 10);
								/* if conversion error or too small/large value, set to default */
								if ((dummy==argv[i]+2) || (SERVER_TIMEOUT<5) || (SERVER_TIMEOUT>90))
									SERVER_TIMEOUT = 25;
							}
                        } else {
                                fprintf (stderr, "Unknown command line switch.\n");
                                print_help ();
                        }
                }
        }

        our_time = time(NULL);

        if (htmloutput == 1) {
                printf ("<body><html>\n");
                printf ("<h1>eDonkey2000 server list</h1><p><b>as of %s local time</b></p>\n", asctime (localtime(&our_time)));
                printf ("<p>click on the server ip to automagically add the server to your server list. "
                        "<font size=-2>(windows users only)</font></p>\n");
                printf ("<p>Alternatively, just download the whole <a href=\"server.met\">server.met</a> file "
                        "to your eDonkey2000 program directory.<br>"
                        "<font size=-2>(right-click on server.met and 'save as...') - <b>Close the donkey before you do this, otherwise it won't read the file and overwrite it with its own data on exit!</b></font></p>\n");
                printf ("<table border width=70%%>");
	        printf ("<tr align=center>"
	                "<th width=32%%> server name </th>"
                        "<th width=32%%> server ip </th>"
                        "<th width=12%%> port </th>"
                        "<th width=12%%> users </th>"
                        "<th width=12%%> files </th>"
        	                "</tr>\n");
        }

        /* if windows version, we have to initialise winsock first */

 #ifdef WINDOWS
        if (WSAStartup(MAKEWORD(1,1), &wsaData) != 0)
                graceful_exit ("WSAStartup failed (winsock couldn't be initialised).\n");
 #endif

        digest_server_file ();

        if (htmloutput == 1) {

                /* print table row for each server found to be still alive */
                for (i=0; i < no_of_useful_servers; i++)
                        print_donkeyserver_info (known_table+i);

                printf ("</table><p><font size=-2>created by ed2k_slist, (p) 2001 by TP Muller</font></html></body>\n");
        }

        if (no_of_useful_servers > 0)
                make_new_server_file ();
        else {
                /* trying to restore our old server.met file */
	        rename ("server-dot-met.backup", "server.met");
                graceful_exit ("None of the servers in the list alive !? I am NOT writing a new server.met file.\nPossibly the connection is down !?\n");
	}

        free_allocated_memory();

        return (0);
}

/* digest_server_file **************************************************************************/

void digest_server_file ()
{
        FILE *file;
        size_t len;

        if ( (file = fopen ("server.met", "rb")) == NULL )
                graceful_exit ("Couldn't find or open server.met file in current directory. Are\n"
                               "you running the donkey from the edonkey program directory?\n");

        if (verbose == 1)
                fprintf (stderr, "Digesting server.met file...\nTimeout is %d seconds...\n", SERVER_TIMEOUT);

        fseek (file, 1, SEEK_SET);                       /* rewind and skip first byte (224=E0) */
        len = fread (&no_of_known_servers, 1, 4, file);  /* read no of known servers (4 bytes)  */

        if (no_of_known_servers == 0)
                graceful_exit ("server.met file says it contains no records.\n"
                               "go to www.edonkey2000.com and add some servers from the serverlist page.\n");

        known_table = calloc (no_of_known_servers, sizeof(donkeyserver));
        if (known_table == NULL)
                graceful_exit ("digest_server_file: not enough available memory.\n");

        no_of_useful_servers = 0;

        for (server_no=0; server_no < no_of_known_servers; server_no++) {
                known_server_useful [server_no] = get_next_known_server (file, (known_table + no_of_useful_servers));

                if (known_server_useful [server_no] == USEFUL) {
	                no_of_useful_servers++;
                } else {
			/* dump server record
			 * by not increasing counter and thus pointer to next record
			 */
                }
        }
        fclose (file);

        if (verbose == 1)
                fprintf (stderr, "Finished digesting server.met file.\n");

}

/* get_next_known_server out of an opened server.met stream **********************************

        returns USEFUL if server connects, otherwise NOT_USEFUL
*/

int get_next_known_server (FILE *file, donkeyserver *dfp)
{
	char 	      ipstr[20], spaces[]="                       ";
	unsigned int  n1,n2,n3,n4;		/*  4 ip sub-numbers                       */
        unsigned int  tags, t;			/*  number of tags in record, counter      */
        size_t 	      len;

        len  = fread (&dfp -> ip_int,1,4, file); /*  get ip as integer (4 bytes)           */
        len += fread (&dfp -> port,1,2,file);    /*  port is only 2 bytes = short          */
        len += fread (&tags, 1, sizeof (unsigned int), file);        /* get number of tags */

	n1 = ((dfp -> ip_int) & (0xFF <<  0)) >> 0;  /* extract 4 ip sub-numbers from      */
	n2 = ((dfp -> ip_int) & (0xFF <<  8)) >> 8;  /* integer ip                         */
	n3 = ((dfp -> ip_int) & (0xFF << 16)) >> 16;
	n4 = ((dfp -> ip_int) & (0xFF << 24)) >> 24;

	int_to_string_with_dot (n1, ipstr + 0            ); /* pack those for ip-sub-nums  */
	int_to_string_with_dot (n2, ipstr + strlen(ipstr)); /* in a string, separated by   */
	int_to_string_with_dot (n3, ipstr + strlen(ipstr)); /* dots                        */
	int_to_string          (n4, ipstr + strlen(ipstr)); /* (no dot after last one)     */

	strcpy (dfp->ip_str, ipstr); /* we got the ip as string: save it in record         */
    strncpy (dfp->name, spaces, 16-strlen(dfp->ip_str));     /* default name empty     */
	dfp->name[16-strlen(dfp->ip_str)]=0x00;					 /* make sure end is there */

	dfp->users = 0;		     /* let those default to zero, just in case we don't   */
	dfp->files = 0;		     /* get values for these from the file                 */
	dfp->ping  = 0;

        server_record_len = len;

        for (t=0; t<tags; t++)	     /* get each tag for our server and store it if useful */
                digest_tag (file, dfp);

        known_server_record_len [server_no] += server_record_len;

	return does_server_connect (dfp);
}

/* digest_tag: get and digest one tag of an opened server.met stream ***************************/

#define MAX_TAG_NAME_STR_LEN 256

void digest_tag (FILE *file, donkeyserver *dfp)
{
        size_t         len;
        unsigned char  tagtype;
        unsigned short name_str_len, value_str_len;
        char           name_str [MAX_TAG_NAME_STR_LEN], value_str [MAX_TAG_NAME_STR_LEN];
        int            tag_int_value;

        len  = fread (&tagtype, 1, 1, file);
        len += fread (&name_str_len, 1, sizeof(short),file);
        len += fread (&name_str, 1, name_str_len, file);
        name_str [name_str_len] = 0x00;                     /* make sure end of string is there */

        if (tagtype == 3)      /* int tag */
                len += fread (&tag_int_value, 1, sizeof(int), file);

        if (tagtype == 2) {    /* string tag */
                len += fread (&value_str_len, 1, sizeof (short), file);
                len += fread (&value_str, 1, value_str_len, file);
                value_str [value_str_len] = 0x00;           /* make sure end of string is there */
        }

        if (name_str_len == 1) {        /* special tag */
                switch (name_str[0]) {
                        case 1: strcpy (dfp -> name, value_str);  /* server name */

                        /* other special tags not interesting:
			       16 = server ip in int format (again)
			       12 = server: ??? (preference? ping?)
			       13 = server: ??? (?)
			       14 = server: ??? (preference? ping?)
 	                */
                }
        }

	if (strcmp(name_str, "users")==0)
		dfp->users = tag_int_value;

	if (strcmp(name_str, "files")==0)
		dfp->files = tag_int_value;

        server_record_len += len;
}

/* print_donkeyserver_info: does what it says ****************************************************/

void print_donkeyserver_info (donkeyserver *dfp)
{
	printf ("<tr align=center>"
		"<td>%s</td>"
		"<td><a href=\"ed2k://|server|%s|%d|\">%s</a></td>",
		dfp->name, dfp->ip_str, dfp->port, dfp->ip_str);

        printf ("<td>%d</td>", dfp->port);

        if (dfp->users > 0)
		printf ("<td>%d</td>", dfp->users);
	else
		printf ("<td></td>");

	if (dfp->files > 0)
		printf ("<td>%d</td>", dfp->files);
	else
		printf ("<td></td>");

	printf ("\n");
}



/* graceful_exit: prints out message and exits gracefully ***************************************/

void graceful_exit (char msg[1024])
{
        fprintf (stderr, msg);
        free_allocated_memory();
        exit(1);
}

/* free_allocated_memory: free all the memory before exit ***************************************/

void free_allocated_memory (void)
{
        if (known_table != NULL)
                free (known_table);

        /* bad programming style: shut down winsock stuff as well here, if windows version */

#ifdef WINDOWS
        WSACleanup();
#endif

}

/* int_to_string */

void int_to_string (int number, char *num_str)
{
	int divider = 10000000, erg, pos=0;

	do {
		erg = number / divider;
		if ((erg>0) || (pos>0)) {
			num_str[pos] = '0' + (char) erg;
			pos++;
		}
		number  = number % divider;
		divider = divider/10;
	} while (divider >= 1);

	if (pos == 0)
		num_str[pos++] = '0';

	num_str[pos] = 0x00;

	return;
}

void int_to_string_with_dot (int number, char *num_str)
{
        int_to_string (number, num_str);
        strcat (num_str, ".");
        return;

}


/* print a small help screen */

void print_help (void)
{
        printf ("\n\ned2k_slist %s: tool for eDonkey2000 (see www.edonkey2000.com)\n", VERSION);
        printf ("(c) 2001 Tim-Philipp Mller - distributed under the GNU Public Licence (GPL)\n\n");

        printf ("takes your eDonkey2000 server.met file (which contains a list of all known\n"
                "servers) and checks for every server in that list if it is still alive. It\n"
                "then creates a new server.met file without the dead servers. The old server.met\n"
                "file is saved as server-dot-met.backup in the same directory.\n\n");
        printf ("A html page with ed2k:// links to all those alive servers can also be\n"
        	"generated on request.\n\n");

        printf ("NB: the servers.met is read by eDonkey only at startup, and updated only\n"
        	"on exit (and not regularly while running). This means you have to run this\n"
                "tool BEFORE you use eDonkey.\n"
		"This tool expects a server.met in its current directory.\n\n");

        printf ("COMMAND LINE SWITCHES:\n"
                "-s    (silent)  supress the output of status information\n"
		"-html (html)    generate a server list as html file to standard output\n"
                "                (redirect to a file like this:'ed2k_slist -html > list.html'\n"
                "-Txx  (timeout) set timeout to xx seconds (xx=5-60s), default: is %d\n\n", DEFAULT_TIMEOUT);

        printf ("USE THIS PROGRAM AT YOUR OWN RISK! Report bugs to ed2k_tools@graffiti.net\n");
	graceful_exit ("");
}


/* copy server.met to server.met.new, leaving out all useless records */

void make_new_server_file ()
{
        FILE *file, *fout;
        int  len, i;
        char buffer[2048];

	remove ("server-dot-met.backup");

	if (verbose==1)
		fprintf (stderr, "Renaming server.met to server-dot-met.backup...\n");

	if (rename ("server.met", "server-dot-met.backup") != 0)
		graceful_exit ("Couldn't rename server.met in server-dot-met.backup\n");

        if ( (file = fopen ("server-dot-met.backup", "rb")) == NULL )
                graceful_exit ("make_new_server_file: couldn't find or open server-dot-met.backup file in current directory.\n"
                               "Are you running the donkey from the edonkey program directory?\n");

        if ( (fout = fopen ("server.met", "wb")) == NULL)
                graceful_exit ("make_new_server_file: couldn't open server.met in write mode.\n");

        if (verbose == 1)
                fprintf (stderr, "Writing new server.met file to donkey directory...\n");

        /* read and write header byte */

        fseek (file, 5, SEEK_SET);          /* skip header (0x0e) + 4 bytes (no of records) */
        fputc   (0x0E, fout);

        /* write number of useful servers */

        fwrite (&no_of_useful_servers, 1, 4, fout);

        /* copy useful server records into new file */

        for (i=0; i < no_of_known_servers; i++) {

                 /* read whole server record */
                len = fread (buffer, 1, known_server_record_len[i], file);
                if (len != known_server_record_len[i])
                        graceful_exit ("make_new_server_file: couldn't read as many bytes as requested !? Strange!\n");

                /* only copy (=write) record if it was useful (i.e. still alive) */

                if (known_server_useful[i] == USEFUL) {
                        len = fwrite (buffer, 1, known_server_record_len[i], fout);
                        if (len != known_server_record_len[i])
                                graceful_exit ("make_new_server_file: couldn't write as many bytes as requested !? Strange!\n");
                }
        }

        fclose (file);
        fclose (fout);

        if (verbose == 1)
                fprintf (stderr, "Finished.\n");
}
