/* WINDOWS NETWORK CONNECTION PART for ed2k_slist
*
*
* --- COPYRIGHT AND LICENCE ---
*
* Copyright (C) 2001 Tim-Philipp Mller (t.i.m@orange.net)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*
*/



/* connect with timeout (windows)
*
* adapted to winsock from linux code probably
* written by Mark Pulford <mark@kyne.com.au>
* found on the LinuxSA mailing list <linuxsa@linuxsa.org.au>
* on Wed, 2 Sep 1998 16:48:06 +0930 (CST)
* Subject: "Re: Socket timeout question"
*
*/

#include <winsock.h>

int do_connect(SOCKET fd,struct sockaddr *remote, int len,
               struct timeval *timeout, int *err)
{
        int 		  ret,back_err;
        fd_set 		  fd_w;
		unsigned long nonblock_flag = 1;

        /* Set non blocking */
	    ioctlsocket (fd, FIONBIO, &nonblock_flag);

        /* This will return immediately */
        *err=connect(fd,remote,len);
        back_err=WSAGetLastError();

        /* return unless the connection was successful or the connect is
           still in progress. */
        if(*err==SOCKET_ERROR && back_err!=WSAEINPROGRESS && back_err!=WSAEWOULDBLOCK) {
                perror("connect");
                *err=errno;
                return -1;
        }

        FD_ZERO(&fd_w);
        FD_SET(fd,&fd_w);

        *err=select(FD_SETSIZE,NULL,&fd_w,NULL,timeout);

        if(*err<0) {
                /*perror("select");*/
                *err=errno;
                return -1;
        }


        /* 0 means it timeout out & no fds changed */
        if(*err==0) {
                closesocket(fd);
                *err=WSAETIMEDOUT;
                return -1;
        }

        /* Get the return code from the connect */
        len=sizeof(ret);
        *err=getsockopt(fd,SOL_SOCKET,SO_ERROR,(char *)&ret,&len);		/* (char *) needed because of definition of func */
        if(*err<0) {
                perror("getsockopt");
                *err=errno;
                return -1;
        }

        /* ret=0 means success, otherwise it contains the errno */
        if (ret!=0) {
                *err=ret;
                return -1;
        }

        *err=0;
        return 0;
}


/* does_server_connect (windows)
*
* returns USEFUL if server connects, otherwise NOT_USEFUL
*
*/

int does_server_connect (donkeyserver *dfp)
{
	SOCKET				 socket_descriptor;	             /* file descriptor for socket        */
	struct sockaddr_in	 pin;
    struct timeval 	     timeout;
	int					 err;

	if (verbose==1)
  		fprintf (stderr, "TRYING %s:%d (%s)... ", dfp->ip_str, dfp->port, dfp->name);

	memset (&pin, 0x00, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_addr.s_addr = inet_addr (dfp->ip_str);
	pin.sin_port = htons(dfp->port);

	if ((socket_descriptor = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if (verbose==1)
			perror("Error opening socket");
        	closesocket (socket_descriptor);
		return (NOT_USEFUL);
	}

        /* set our timeout value */

        timeout.tv_sec  = SERVER_TIMEOUT;    /* seconds                                        */
        timeout.tv_usec = 0;     			 /* micro-seconds (1 us = 1/1000 ms = 1/1000000 s) */

	if (do_connect(socket_descriptor, (void *)&pin, sizeof(pin), &timeout, &err) == -1) {
		if (verbose==1)
			fprintf (stderr,"timed out or connection refused\n");
                closesocket (socket_descriptor);
		return (NOT_USEFUL);
	}

	/* connection must have been successful */

	if (verbose==1)
		fprintf (stderr,"successfully connected!\n");

	closesocket (socket_descriptor);

	return (USEFUL);;
}

/* http_server_connect (windows)
*
* returns file descriptor if connect successful, otherwise -1
*
*/

int http_server_connect (char servername[256])
{
	SOCKET				 socket_descriptor;	             /* file descriptor for socket        */
	struct sockaddr_in	 pin;
	struct hostent 		 *hp;

	memset (&pin, 0x00, sizeof(pin));
	pin.sin_family = AF_INET;
	pin.sin_addr.s_addr = htonl(INADDR_ANY);
	pin.sin_port = htons(80);

	if (verbose==1)
  		fprintf (stderr, "TRYING TO RESOLVE %s...", servername);

	hp = gethostbyname(servername);             /* get our address info */
  	if (hp == NULL) {                           /* can't be resolved */
		fprintf (stderr, "failed to resolve hostname! (error %d)\n", WSAGetLastError());
    	return(-1);
	}

	fprintf (stderr, "ok.\n");
	memcpy((char *)&pin.sin_addr, hp->h_addr, hp->h_length);		/* resolve ok */

	if (verbose==1)
  		fprintf (stderr, "TRYING TO CONNECT TO %s:80...", servername);


	if ((socket_descriptor = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if (verbose==1)
			perror("Error opening socket");
        	closesocket (socket_descriptor);
		return (-1);
	}

	if (connect(socket_descriptor, (void *)&pin, sizeof(pin)) == -1) {
		if (verbose==1)
			fprintf (stderr,"timed out or connection refused\n");
                closesocket (socket_descriptor);
		return (-1);
	}

	/* connection must have been successful */

	if (verbose==1)
		fprintf (stderr,"connected!\n");

	return (socket_descriptor);
}


